/*
 * Copyright (C) 2023  Valentin Lorentz
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License version 3,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

use crate::widgets::OverlappableWidget;
use ratatui::prelude::*;
use ratatui::widgets::*;

pub struct Divider<'a> {
  pub line_type: BorderType,
  pub text: Paragraph<'a>,
}

impl<'a> Divider<'a> {
  pub fn new(text: Paragraph<'a>) -> Self {
    Divider {
      line_type: BorderType::default(),
      text,
    }
  }
}

impl<'a> OverlappableWidget for Divider<'a> {
  fn height(&self, width: u16) -> u64 {
    1
  }

  fn render_overlap(
    self,
    area: ratatui::layout::Rect,
    buf: &mut ratatui::buffer::Buffer,
  ) -> (u16, u16) {
    if area.height == 0 {
      return (area.width, 0);
    }
    let symbol = BorderType::border_symbols(self.line_type).horizontal_top;
    for x in 0..area.width {
      buf
        .get_mut(area.x + x, area.y + area.height - 1)
        .set_symbol(symbol);
    }

    self.text.render(
      Rect {
        y: area.y + area.height - 1,
        height: 1,
        ..area
      },
      buf,
    );

    (area.width, 1)
  }
}
